@@ -32,10 +32,12 @@
 #include <yelp-util.h>
 #include <libgnome/gnome-i18n.h>
 
+#include <sys/stat.h>
 #include <sys/types.h>
 #include <string.h>
 #include <stdio.h>
 #include <dirent.h>
+#include <unistd.h>
 
 struct TreeNode {
 	char *name;
@@ -159,7 +161,8 @@ yelp_man_populate_tree_for_subdir (GHashTable *section_hash,
 		g_assert (node != NULL);
 		
 		node->pages = g_list_prepend (node->pages, 
-					      yelp_section_new (titlebuf, uribuf, 
+					      yelp_section_new (YELP_SECTION_DOCUMENT,
+								titlebuf, uribuf, 
 								NULL, NULL));
 		g_free (manname);
 		g_free (section);
@@ -437,7 +440,7 @@ struct TreeData root_children[] = {
 
 
 struct TreeData root_data = {
-	N_("Manual Pages"),
+	N_("man"),
 	root_children,
 	NULL
 };
@@ -505,23 +508,24 @@ yelp_man_free_initial_tree (struct TreeNode *node)
 }
 
 static void
-yelp_man_push_initial_tree (struct TreeNode *node, GtkTreeStore *store, GtkTreeIter *parent)
+yelp_man_push_initial_tree (struct TreeNode *node, GNode *parent)
 {
 	struct TreeNode *child;
-	YelpSection *page;
-	GtkTreeIter *iter;
-	GList *l;
-
-	iter = yelp_util_contents_add_section (store, parent, 
-					       yelp_section_new (node->name, NULL, 
-								 NULL, NULL));
+	YelpSection     *page;
+	GList           *l;
+	GNode           *man_node;
+	
+	man_node = g_node_append_data (parent, 
+				       yelp_section_new (YELP_SECTION_CATEGORY,
+							 node->name, NULL, 
+							 NULL, NULL));
 	
 	l = node->tree_nodes;
 	while (l) {
 		child = l->data;
 		l = l->next;
 
-		yelp_man_push_initial_tree (child, store, iter);
+		yelp_man_push_initial_tree (child, man_node);
 	}
 	
 	l = node->pages;
@@ -529,55 +533,61 @@ yelp_man_push_initial_tree (struct TreeNode *node, GtkTreeStore *store, GtkTreeI
 		page = l->data;
 		l = l->next;
 
-		yelp_util_contents_add_section (store, iter, page);
+		g_node_append_data (man_node, page);
 	}
 }
 
 gboolean
-yelp_man_init (GtkTreeStore *store)
+yelp_man_init (GNode *tree)
 {
-	FILE *fh;
-	char aline[1024];
-	char **manpath = NULL;
-	int i;
-	GHashTable *section_hash;
-	struct TreeNode *root;
+	gchar            *manpath = NULL;
+ 	char            **manpathes = NULL;
+	GHashTable       *section_hash;
+	struct TreeNode  *root;
+	struct stat       stat_dir1;
+	struct stat       stat_dir2;
+	int               i;
 
 	/* Go through all the man pages:
 	 * 1. Determine the places to search (run 'manpath').
-	 * 2. Go through all subdirectories to find individual files.
-	 * 3. For each file, add it onto the tree at the right place.
+	 * 2. If that is not found, use MANPATH environment variable.
+	 * 3. Go through all subdirectories to find individual files.
+	 * 4. For each file, add it onto the tree at the right place.
 	 */
 
 	section_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	
 	root = yelp_man_make_initial_tree (&root_data, section_hash);
 
-	fh = popen ("manpath", "r");
-	g_return_val_if_fail (fh, FALSE);
-
-	if (fgets (aline, sizeof (aline), fh)) {
-		g_strstrip (aline);
-		manpath = g_strsplit (aline, ":", -1);
-	} else {
-		g_warning ("Couldn't get manpath");
+	if (!g_spawn_command_line_sync ("manpath", &manpath, NULL, NULL, NULL)) {
+		g_print ("manpath not found, looking for MANPATH env\n");
+		manpath = g_strdup (g_getenv ("MANPATH"));
 	}
-	pclose (fh);
 
-	i = 0;
 	if (manpath) {
-		for (; manpath[i]; i++)
-			yelp_man_populate_tree_for_dir (section_hash, manpath[i]);
-	}
-	if (!manpath || !manpath[0]) {
-		yelp_man_populate_tree_for_dir (section_hash, "/usr/man");
-		yelp_man_populate_tree_for_dir (section_hash, "/usr/share/man");
-	}
+		g_strstrip (manpath);
+		manpathes = g_strsplit (manpath, ":", -1);
+		g_free (manpath);
 
+		for (i = 0; manpathes[i]; i++) {
+			yelp_man_populate_tree_for_dir (section_hash, 
+							manpathes[i]);
+		}
+	} else {
+		stat ("/usr/man", &stat_dir1);
+		stat ("/usr/share/man", &stat_dir2);
+		
+		yelp_man_populate_tree_for_dir (section_hash, "/usr/man");
+
+		if (stat_dir1.st_ino != stat_dir2.st_ino) {
+			yelp_man_populate_tree_for_dir (section_hash, 
+							"/usr/share/man");
+		}
+	}
 
 	yelp_man_cleanup_initial_tree (root);
 	
-	yelp_man_push_initial_tree (root, store, NULL);
+	yelp_man_push_initial_tree (root, tree);
 	
 	yelp_man_free_initial_tree (root);
 	g_hash_table_destroy (section_hash);
